
-- Create the TAP_UPLOAD schema:

\echo '[INFO: Creating TAP_UPLOAD...]'

CREATE SCHEMA "TAP_UPLOAD";

\echo '[INFO: TAP_UPLOAD created]'

--------------------------------------------------------------------------------

-- Create the TAP_SCHEMA schema:

\echo '[INFO: Creating TAP_SCHEMA...]'

CREATE SCHEMA "TAP_SCHEMA";

--------------------------------------------------------------------------------

-- Create the 'schemas' table:

CREATE TABLE "TAP_SCHEMA"."schemas" (
    "schema_index" SMALLINT,
    "schema_name"  VARCHAR,
    "description"  VARCHAR,
    "utype"        VARCHAR,
    "dbname"       VARCHAR,
    PRIMARY KEY("schema_name")
);

INSERT INTO  "TAP_SCHEMA"."schemas" VALUES (1, 'TAP_SCHEMA', 'Set of tables listing and describing the schemas, tables and columns published in this TAP service.', NULL, NULL);

--------------------------------------------------------------------------------

-- Create the 'tables' table:

CREATE TABLE "TAP_SCHEMA"."tables" (
    "table_index" SMALLINT,
    "schema_name" VARCHAR,
    "table_name"  VARCHAR,
    "table_type"  VARCHAR,
    "description" VARCHAR,
    "utype"       VARCHAR,
    "dbname"      VARCHAR,
    PRIMARY KEY("table_name"),
    CONSTRAINT fk_tables_schema_name
        FOREIGN KEY(schema_name)
            REFERENCES "TAP_SCHEMA".schemas(schema_name)
            ON DELETE CASCADE
);

INSERT INTO "TAP_SCHEMA"."tables" VALUES (1, 'TAP_SCHEMA', 'TAP_SCHEMA.schemas'    , 'table', 'List of schemas published in this TAP service.'                                                                                                                                 , NULL, NULL);
INSERT INTO "TAP_SCHEMA"."tables" VALUES (2, 'TAP_SCHEMA', 'TAP_SCHEMA.tables'     , 'table', 'List of tables published in this TAP service.'                                                                                                                                  , NULL, NULL);
INSERT INTO "TAP_SCHEMA"."tables" VALUES (3, 'TAP_SCHEMA', 'TAP_SCHEMA.columns'    , 'table', 'List of columns of all tables listed in TAP_SCHEMA.TABLES and published in this TAP service.'                                                                                   , NULL, NULL);
INSERT INTO "TAP_SCHEMA"."tables" VALUES (4, 'TAP_SCHEMA', 'TAP_SCHEMA.keys'       , 'table', 'List all foreign keys but provides just the tables linked by the foreign key. To know which columns of these tables are linked, see in TAP_SCHEMA.key_columns using the key_id.', NULL, NULL);
INSERT INTO "TAP_SCHEMA"."tables" VALUES (5, 'TAP_SCHEMA', 'TAP_SCHEMA.key_columns', 'table', 'List all foreign keys but provides just the columns linked by the foreign key. To know the table of these columns, see in TAP_SCHEMA.keys using the key_id.'                    , NULL, NULL);

--------------------------------------------------------------------------------

-- Create the 'columns' table:

CREATE TABLE "TAP_SCHEMA"."columns" (
    "column_index" SMALLINT,
    "table_name"   VARCHAR,
    "column_name"  VARCHAR,
    "description"  VARCHAR,
    "unit"         VARCHAR,
    "ucd"          VARCHAR,
    "utype"        VARCHAR,
    "datatype"     VARCHAR,
    "size"         INTEGER,
    "principal"    SMALLINT,
    "indexed"      SMALLINT,
    "std"          SMALLINT,
    "dbname"       VARCHAR,
    PRIMARY KEY("table_name","column_name"),
    CONSTRAINT fk_columns_table_name
        FOREIGN KEY(table_name)
            REFERENCES "TAP_SCHEMA".tables(table_name)
            ON DELETE CASCADE
);

INSERT INTO "TAP_SCHEMA"."columns" VALUES (1,  'TAP_SCHEMA.schemas'    , 'schema_name'  , 'schema name, possibly qualified'                , NULL, NULL, NULL, 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (2,  'TAP_SCHEMA.schemas'    , 'description'  , 'brief description of schema'                    , NULL, NULL, NULL, 'VARCHAR', -1, 0, 0, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (3,  'TAP_SCHEMA.schemas'    , 'utype'        , 'UTYPE if schema corresponds to a data model'    , NULL, NULL, NULL, 'VARCHAR', -1, 0, 0, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (1,  'TAP_SCHEMA.tables'     , 'schema_name'  , 'the schema name from TAP_SCHEMA.schemas'        , NULL, NULL, NULL, 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (2,  'TAP_SCHEMA.tables'     , 'table_name'   , 'table name as it should be used in queries'     , NULL, NULL, NULL, 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (3,  'TAP_SCHEMA.tables'     , 'table_type'   , 'one of: table, view'                            , NULL, NULL, NULL, 'VARCHAR', -1, 0, 0, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (4,  'TAP_SCHEMA.tables'     , 'description'  , 'brief description of table'                     , NULL, NULL, NULL, 'VARCHAR', -1, 0, 0, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (5,  'TAP_SCHEMA.tables'     , 'utype'        , 'UTYPE if table corresponds to a data model'     , NULL, NULL, NULL, 'VARCHAR', -1, 0, 0, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (1,  'TAP_SCHEMA.columns'    , 'table_name'   , 'table name from TAP_SCHEMA.tables'              , NULL, NULL, NULL, 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (2,  'TAP_SCHEMA.columns'    , 'column_name'  , 'column name'                                    , NULL, NULL, NULL, 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (3,  'TAP_SCHEMA.columns'    , 'description'  , 'brief description of column'                    , NULL, NULL, NULL, 'VARCHAR', -1, 0, 0, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (4,  'TAP_SCHEMA.columns'    , 'unit'         , 'unit in VO standard format'                     , NULL, NULL, NULL, 'VARCHAR', -1, 0, 0, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (5,  'TAP_SCHEMA.columns'    , 'ucd'          , 'UCD of column if any'                           , NULL, NULL, NULL, 'VARCHAR', -1, 0, 0, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (6,  'TAP_SCHEMA.columns'    , 'utype'        , 'UTYPE of column if any'                         , NULL, NULL, NULL, 'VARCHAR', -1, 0, 0, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (7,  'TAP_SCHEMA.columns'    , 'datatype'     , 'ADQL datatype as in section 2.5'                , NULL, NULL, NULL, 'VARCHAR', -1, 0, 0, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (8,  'TAP_SCHEMA.columns'    , 'size'         , 'length of variable length datatypes'            , NULL, NULL, NULL, 'INTEGER', -1, 0, 0, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (9,  'TAP_SCHEMA.columns'    , 'principal'    , 'a principal column; 1 means true, 0 means false', NULL, NULL, NULL, 'INTEGER', -1, 0, 0, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (10, 'TAP_SCHEMA.columns'    , 'indexed'      , 'an indexed column; 1 means true, 0 means false' , NULL, NULL, NULL, 'INTEGER', -1, 0, 0, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (11, 'TAP_SCHEMA.columns'    , 'std'          , 'a standard column; 1 means true, 0 means false' , NULL, NULL, NULL, 'INTEGER', -1, 0, 0, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (1,  'TAP_SCHEMA.keys'       , 'key_id'       , 'unique key identifier'                          , NULL, NULL, NULL, 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (2,  'TAP_SCHEMA.keys'       , 'from_table'   , 'fully qualified table name'                     , NULL, NULL, NULL, 'VARCHAR', -1, 0, 0, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (3,  'TAP_SCHEMA.keys'       , 'target_table' , 'fully qualified table name'                     , NULL, NULL, NULL, 'VARCHAR', -1, 0, 0, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (4,  'TAP_SCHEMA.keys'       , 'description'  , 'description of this key'                        , NULL, NULL, NULL, 'VARCHAR', -1, 0, 0, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (5,  'TAP_SCHEMA.keys'       , 'utype'        , 'utype of this key'                              , NULL, NULL, NULL, 'VARCHAR', -1, 0, 0, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (1,  'TAP_SCHEMA.key_columns', 'key_id'       , 'unique key identifier'                          , NULL, NULL, NULL, 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (2,  'TAP_SCHEMA.key_columns', 'from_column'  , 'key column name in the from_table'              , NULL, NULL, NULL, 'VARCHAR', -1, 0, 0, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (3,  'TAP_SCHEMA.key_columns', 'target_column', 'key column name in the target_table'            , NULL, NULL, NULL, 'VARCHAR', -1, 0, 0, 1, NULL);

--------------------------------------------------------------------------------

-- Create the 'keys' table:

CREATE TABLE "TAP_SCHEMA"."keys" (
    "key_id"       VARCHAR,
    "from_table"   VARCHAR,
    "target_table" VARCHAR,
    "description"  VARCHAR,
    "utype"        VARCHAR,
    PRIMARY KEY("key_id"),
    CONSTRAINT fk_keys_from_table
        FOREIGN KEY(from_table)
            REFERENCES "TAP_SCHEMA".tables(table_name)
            ON DELETE CASCADE,
    CONSTRAINT fk_keys_target_table
        FOREIGN KEY(target_table)
            REFERENCES "TAP_SCHEMA".tables(table_name)
            ON DELETE CASCADE
);

INSERT INTO "TAP_SCHEMA"."keys" VALUES ('fk_tables_schema_name', 'TAP_SCHEMA.tables'     , 'TAP_SCHEMA.schemas', NULL, NULL);
INSERT INTO "TAP_SCHEMA"."keys" VALUES ('fk_columns_table_name', 'TAP_SCHEMA.columns'    , 'TAP_SCHEMA.tables' , NULL, NULL);
INSERT INTO "TAP_SCHEMA"."keys" VALUES ('fk_keys_from_table'   , 'TAP_SCHEMA.keys'       , 'TAP_SCHEMA.tables' , NULL, NULL);
INSERT INTO "TAP_SCHEMA"."keys" VALUES ('fk_keys_target_table' , 'TAP_SCHEMA.keys'       , 'TAP_SCHEMA.tables' , NULL, NULL);
INSERT INTO "TAP_SCHEMA"."keys" VALUES ('fk_key_columns_key_id', 'TAP_SCHEMA.key_columns', 'TAP_SCHEMA.keys'   , NULL, NULL);

--------------------------------------------------------------------------------

-- Create the 'key_columns' table:

CREATE TABLE "TAP_SCHEMA"."key_columns" (
    "key_id"        VARCHAR,
    "from_column"   VARCHAR,
    "target_column" VARCHAR,
    PRIMARY KEY("key_id", "from_column"),
    CONSTRAINT fk_key_columns_key_id
        FOREIGN KEY(key_id)
            REFERENCES "TAP_SCHEMA".keys(key_id)
            ON DELETE CASCADE
);

INSERT INTO "TAP_SCHEMA"."key_columns" VALUES('fk_tables_schema_name', 'schema_name' , 'schema_name');
INSERT INTO "TAP_SCHEMA"."key_columns" VALUES('fk_columns_table_name', 'table_name'  , 'table_name' );
INSERT INTO "TAP_SCHEMA"."key_columns" VALUES('fk_keys_from_table'   , 'from_table'  , 'table_name' );
INSERT INTO "TAP_SCHEMA"."key_columns" VALUES('fk_keys_target_table' , 'target_table', 'table_name' );
INSERT INTO "TAP_SCHEMA"."key_columns" VALUES('fk_key_columns_key_id', 'key_id'      , 'key_id'     );

\echo '[INFO: TAP_SCHEMA created]'
