
\echo '[INFO: Creating rr tables...]'

-- Create the schema 'rr':

CREATE SCHEMA rr;
INSERT INTO "TAP_SCHEMA".schemas VALUES(2, 'rr', 'RegTAP or Relational Registry (rr) schema', 'ivo://ivoa.net/std/RegTAP#1.1', NULL);

--------------------------------------------------------------------------------

-- Create the table 'resource':

CREATE TABLE rr.resource (
    ivoid            VARCHAR,
    res_type         VARCHAR,
    created          TIMESTAMP,
    short_name       VARCHAR,
    res_title        VARCHAR,
    updated          TIMESTAMP,
    content_level    VARCHAR,
    res_description  VARCHAR,
    reference_url    VARCHAR,
    creator_seq      VARCHAR,
    content_type     VARCHAR,
    source_format    VARCHAR,
    source_value     VARCHAR,
    res_version      VARCHAR,
    region_of_regard REAL,
    waveband         VARCHAR,
    rights           VARCHAR,
    rights_uri       VARCHAR,
    PRIMARY KEY(ivoid)
);

-- Force some fields in lowercase:

CREATE OR REPLACE FUNCTION rr.lower_resource() RETURNS trigger AS
$BODY$
BEGIN
    NEW.ivoid         := lower(OLD.ivoid);
    NEW.res_type      := lower(OLD.res_type);
    NEW.content_level := lower(OLD.content_level);
    NEW.content_type  := lower(OLD.content_type);
    NEW.source_format := lower(OLD.source_format);
    NEW.waveband      := lower(OLD.waveband);
END
$BODY$
LANGUAGE plpgsql;

CREATE TRIGGER trg_resource BEFORE INSERT OR UPDATE ON rr.resource
    FOR EACH ROW EXECUTE PROCEDURE rr.lower_resource();

-- Update TAP_SCHEMA:

INSERT INTO "TAP_SCHEMA"."tables" VALUES(1, 'rr', 'rr.resource', 'table', 'The resources (like services, data collections, organizations) present in this registry.', 'xpath:/', NULL);

INSERT INTO "TAP_SCHEMA"."columns" VALUES (1,  'rr.resource', 'ivoid'           , 'Unambiguous reference to the resource conforming to the IVOA standard for identifiers.'                                                                                              , NULL , NULL, 'xpath:identifier'             , 'VARCHAR'  , -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (2,  'rr.resource', 'res_type'        , 'Resource type (something like vs:datacollection, vs:catalogservice, etc).'                                                                                                           , NULL , NULL, 'xpath:@xsi:type'              , 'VARCHAR'  , -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (3,  'rr.resource', 'created'         , 'The UTC date and time this resource metadata description was created.'                                                                                                               , NULL , NULL, 'xpath:@created'               , 'TIMESTAMP', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (4,  'rr.resource', 'short_name'      , 'A short name or abbreviation given to something, for presentation in space-constrained fields (up to 16 characters).'                                                                , NULL , NULL, 'xpath:shortName'              , 'VARCHAR'  , -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (5,  'rr.resource', 'res_title'       , 'The full name given to the resource.'                                                                                                                                                , NULL , NULL, 'xpath:title'                  , 'VARCHAR'  , -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (6,  'rr.resource', 'updated'         , 'The UTC date this resource metadata description was last updated.'                                                                                                                   , NULL , NULL, 'xpath:@udpated'               , 'TIMESTAMP', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (7,  'rr.resource', 'content_level'   , 'A hash-separated list of content levels specifying the intended audience.'                                                                                                           , NULL , NULL, 'xpath:content/contentLevel'   , 'VARCHAR'  , -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (8,  'rr.resource', 'res_description' , 'An account of the nature of the resource.'                                                                                                                                           , NULL , NULL, 'xpath:content/description'    , 'VARCHAR'  , -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (9,  'rr.resource', 'reference_url'   , 'URL pointing to a human-readable document describing this resource.'                                                                                                                 , NULL , NULL, 'xpath:content/referenceURL'   , 'VARCHAR'  , -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (10, 'rr.resource', 'creator_seq'     , 'The creator(s) of the resource in the order given by the resource record author, separated by semicolons.'                                                                           , NULL , NULL, 'xpath:curation/creator/name'  , 'VARCHAR'  , -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (11, 'rr.resource', 'content_type'    , 'A hash-separated list of natures or genres of the content of the resource.'                                                                                                          , NULL , NULL, 'xpath:content/type'           , 'VARCHAR'  , -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (12, 'rr.resource', 'source_format'   , 'The format of source_value. This, in particular, can be "bibcode".'                                                                                                                  , NULL , NULL, 'xpath:content/source/@format' , 'VARCHAR'  , -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (13, 'rr.resource', 'source_value'    , 'A bibliographic reference from which the present resource is derived or extracted.'                                                                                                  , NULL , NULL, 'xpath:content/source'         , 'VARCHAR'  , -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (14, 'rr.resource', 'res_version'     , 'Label associated with creation or availablilty of a version of a resource.'                                                                                                          , NULL , NULL, 'xpath:curation/version'       , 'VARCHAR'  , -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (15, 'rr.resource', 'region_of_regard', 'A single numeric value representing the angle, given in decimal degrees, by which a positional query against this resource should be "blurred" in order to get an appropriate match.', 'deg', NULL, 'xpath:coverage/regionOfRegard', 'REAL'     , -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (16, 'rr.resource', 'waveband'        , 'A hash-separated list of regions of the electro-magnetic spectrum that the resource''s spectral coverage overlaps with.'                                                             , NULL , NULL, 'xpath:coverage/waveband'      , 'VARCHAR'  , -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (17, 'rr.resource', 'rights'          , 'A statement of usage conditions (license, attribution, embargo, etc).'                                                                                                               , NULL , NULL, 'xpath:rights'                 , 'VARCHAR'  , -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (18, 'rr.resource', 'rights_uri'      , 'A URI identifying a license the data is made available under.'                                                                                                                       , NULL , NULL, 'xpath:rights/@rightsURI'      , 'VARCHAR'  , -1, 1, 1, 1, NULL);

--------------------------------------------------------------------------------

-- Create the table 'res_role':

CREATE TABLE rr.res_role (
    ivoid          VARCHAR,
    role_name      VARCHAR,
    role_ivoid     VARCHAR,
    street_address VARCHAR,
    email          VARCHAR,
    telephone      VARCHAR,
    logo           VARCHAR,
    base_role      VARCHAR,
    PRIMARY KEY (ivoid),
    CONSTRAINT fk_res_role_ivoid
        FOREIGN KEY(ivoid)
            REFERENCES rr.resource(ivoid)
            ON DELETE CASCADE
);

-- Force some fields in lowercase:

CREATE OR REPLACE FUNCTION rr.lower_res_role() RETURNS trigger AS
$BODY$
BEGIN
    NEW.ivoid      := lower(OLD.ivoid);
    NEW.role_ivoid := lower(OLD.role_ivoid);
    NEW.base_role  := lower(OLD.base_role);
END
$BODY$
LANGUAGE plpgsql;

CREATE TRIGGER trg_res_role BEFORE INSERT OR UPDATE ON rr.res_role
    FOR EACH ROW EXECUTE PROCEDURE rr.lower_res_role();

-- Create some indexes:

CREATE INDEX idx_res_role_name ON rr.res_role (role_name);

-- Update TAP_SCHEMA:

INSERT INTO "TAP_SCHEMA"."tables" VALUES(2, 'rr', 'rr.res_role', 'table', 'The resources (like services, data collections, organizations) present in this registry.', NULL, NULL);

INSERT INTO "TAP_SCHEMA"."columns" VALUES (1,  'rr.res_role', 'ivoid'         , 'The parent resource.'                                                                       , NULL , NULL, 'xpath:/identifier', 'VARCHAR'  , -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (2,  'rr.res_role', 'role_name'     , 'The real-world name or title of a person or organization.'                                  , NULL , NULL, NULL               , 'VARCHAR'  , -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (3,  'rr.res_role', 'role_ivoid'    , 'An IVOA identifier of a person or organization.'                                            , NULL , NULL, NULL               , 'VARCHAR'  , -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (4,  'rr.res_role', 'street_address', 'A mailing address for a person or organization.'                                            , NULL , NULL, NULL               , 'VARCHAR'  , -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (5,  'rr.res_role', 'email'         , 'An email address the entity can be reached at.'                                             , NULL , NULL, NULL               , 'VARCHAR'  , -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (6,  'rr.res_role', 'telephone'     , 'A telephone number the entity can be reached at.'                                           , NULL , NULL, NULL               , 'VARCHAR'  , -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (7,  'rr.res_role', 'logo'          , 'URL pointing to a graphical logo, which may be used to help identify the entity.'           , NULL , NULL, NULL               , 'VARCHAR'  , -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (8,  'rr.res_role', 'base_role'     , 'The role played by this entity; this is one of contact, publisher, contributor, or creator.', NULL , NULL, NULL               , 'VARCHAR'  , -1, 1, 1, 1, NULL);

INSERT INTO "TAP_SCHEMA"."keys" VALUES ('fk_res_role_ivoid', 'rr.res_role', 'rr.resource', NULL, NULL);

INSERT INTO "TAP_SCHEMA"."key_columns" VALUES ('fk_res_role_ivoid', 'ivoid', 'ivoid');

--------------------------------------------------------------------------------

-- Create the table 'res_subject':

CREATE TABLE rr.res_subject (
    ivoid VARCHAR,
    res_subject VARCHAR,
    PRIMARY KEY (ivoid),
    CONSTRAINT fk_res_subject_ivoid
        FOREIGN KEY(ivoid)
            REFERENCES rr.resource(ivoid)
            ON DELETE CASCADE
);

-- Force some fields in lowercase:

CREATE OR REPLACE FUNCTION rr.lower_res_subject() RETURNS trigger AS
$BODY$
BEGIN
    NEW.ivoid := lower(OLD.ivoid);
END
$BODY$
LANGUAGE plpgsql;

CREATE TRIGGER trg_res_subject BEFORE INSERT OR UPDATE ON rr.res_subject
    FOR EACH ROW EXECUTE PROCEDURE rr.lower_res_subject();

-- Create some indexes:

CREATE INDEX idx_res_subject ON rr.res_subject (LOWER(res_subject));

-- Update TAP_SCHEMA:

INSERT INTO "TAP_SCHEMA"."tables" VALUES(3, 'rr', 'rr.res_subject', 'table', 'Topics, object types, or other descriptive keywords about the resource.', 'xpath:/content/', NULL);

INSERT INTO "TAP_SCHEMA"."columns" VALUES (1,  'rr.res_subject', 'ivoid'      , 'The parent resource.'                                                   , NULL, NULL, 'xpath:/identifier', 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (2,  'rr.res_subject', 'res_subject', 'Topics, object types, or other descriptive keywords about the resource.', NULL, NULL, 'xpath:subject'    , 'VARCHAR', -1, 1, 1, 1, NULL);

INSERT INTO "TAP_SCHEMA"."keys" VALUES ('fk_res_subject_ivoid', 'rr.res_subject', 'rr.resource', NULL, NULL);

INSERT INTO "TAP_SCHEMA"."key_columns" VALUES ('fk_res_subject_ivoid', 'ivoid', 'ivoid');

--------------------------------------------------------------------------------

-- Create the table 'capability':
CREATE TABLE rr.capability (
    ivoid           VARCHAR,
    cap_index       INTEGER,
    cap_type        VARCHAR,
    cap_description VARCHAR,
    standard_id     VARCHAR,
    PRIMARY KEY (ivoid, cap_index),
    CONSTRAINT fk_capability_ivoid
        FOREIGN KEY(ivoid)
            REFERENCES rr.resource(ivoid)
            ON DELETE CASCADE
);

-- Force some fields in lowercase:

CREATE OR REPLACE FUNCTION rr.lower_capability() RETURNS trigger AS
$BODY$
BEGIN
    NEW.ivoid       := lower(OLD.ivoid);
    NEW.cap_type    := lower(OLD.cap_type);
    NEW.standard_id := lower(OLD.standard_id);
END
$BODY$
LANGUAGE plpgsql;

CREATE TRIGGER trg_capability BEFORE INSERT OR UPDATE ON rr.capability
    FOR EACH ROW EXECUTE PROCEDURE rr.lower_capability();

-- Create some indexes:

CREATE INDEX idx_capability_cap_type    ON rr.capability (cap_type);
CREATE INDEX idx_capability_standard_id ON rr.capability (standard_id);

-- Update TAP_SCHEMA:

INSERT INTO "TAP_SCHEMA"."tables" VALUES(4, 'rr', 'rr.capability', 'table', 'Pieces of behaviour of a resource.', 'xpath:/capability', NULL);

INSERT INTO "TAP_SCHEMA"."columns" VALUES (1,  'rr.capability', 'ivoid'          , 'The parent resource.'                                                                                                                                               , NULL, NULL, 'xpath:/identifier', 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (2,  'rr.capability', 'cap_index'      , 'An arbitrary identifier of this capability within the resource.'                                                                                                    , NULL, NULL, NULL               , 'INTEGER', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (3,  'rr.capability', 'cap_type'       , 'The type of capability covered here. If looking for endpoints implementing a certain standard, you should not use this column but rather match against standard_id.', NULL, NULL, 'xpath:@xsi:type'  , 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (4,  'rr.capability', 'cap_description', 'A human-readable description of what this capability provides as part of the over-all service.'                                                                     , NULL, NULL, 'xpath:description', 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (5,  'rr.capability', 'standard_id'    , 'A URI for a standard this capability conforms to.'                                                                                                                  , NULL, NULL, 'xpath:@standardID', 'VARCHAR', -1, 1, 1, 1, NULL);

INSERT INTO "TAP_SCHEMA"."keys" VALUES ('fk_capability_ivoid', 'rr.capability', 'rr.resource', NULL, NULL);

INSERT INTO "TAP_SCHEMA"."key_columns" VALUES ('fk_capability_ivoid', 'ivoid', 'ivoid');

--------------------------------------------------------------------------------

-- Create the table 'res_schema':
CREATE TABLE rr.res_schema (
    ivoid              VARCHAR,
    schema_index       INTEGER,
    schema_description VARCHAR,
    schema_name        VARCHAR,
    schema_title       VARCHAR,
    schema_utype       VARCHAR,
    PRIMARY KEY (ivoid, schema_index),
    CONSTRAINT fk_res_schema_ivoid
        FOREIGN KEY(ivoid)
            REFERENCES rr.resource(ivoid)
            ON DELETE CASCADE
);

-- Force some fields in lowercase:

CREATE OR REPLACE FUNCTION rr.lower_res_schema() RETURNS trigger AS
$BODY$
BEGIN
    NEW.ivoid        := lower(OLD.ivoid);
    NEW.schema_name  := lower(OLD.schema_name);
    NEW.schema_utype := lower(OLD.schema_utype);
END
$BODY$
LANGUAGE plpgsql;

CREATE TRIGGER trg_res_schema BEFORE INSERT OR UPDATE ON rr.res_schema
    FOR EACH ROW EXECUTE PROCEDURE rr.lower_res_schema();

-- Create some indexes:

CREATE INDEX idx_res_schema_schema_name ON rr.res_schema (schema_name);

-- Update TAP_SCHEMA:

INSERT INTO "TAP_SCHEMA"."tables" VALUES(5, 'rr', 'rr.res_schema', 'table', 'Sets of tables related to resources.', 'xpath:/tableset/schema/', NULL);

INSERT INTO "TAP_SCHEMA"."columns" VALUES (1,  'rr.res_schema', 'ivoid'             , 'The parent resource.'                                                                            , NULL, NULL, 'xpath:/identifier', 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (2,  'rr.res_schema', 'schema_index'      , 'An arbitrary identifier for the res_schema rows belonging to a resource.'                        , NULL, NULL, NULL               , 'INTEGER', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (3,  'rr.res_schema', 'schema_description', 'A free text description of the tableset explaining in general how all of the tables are related.', NULL, NULL, 'xpath:description', 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (4,  'rr.res_schema', 'schema_name'       , 'A name for the set of tables.'                                                                   , NULL, NULL, 'xpath:name'       , 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (5,  'rr.res_schema', 'schema_title'      , 'A descriptive, human-interpretable name for the table set.'                                      , NULL, NULL, 'xpath:title'      , 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (6,  'rr.res_schema', 'schema_utype'      , 'An identifier for a concept in a data model that the data in this schema as a whole represent.'  , NULL, NULL, 'xpath:utype'      , 'VARCHAR', -1, 1, 1, 1, NULL);

INSERT INTO "TAP_SCHEMA"."keys" VALUES ('fk_res_schema_ivoid', 'rr.res_schema', 'rr.resource', NULL, NULL);

INSERT INTO "TAP_SCHEMA"."key_columns" VALUES ('fk_res_schema_ivoid', 'ivoid', 'ivoid');

--------------------------------------------------------------------------------

-- Create the table 'res_table':
CREATE TABLE rr.res_table (
    ivoid             VARCHAR,
    schema_index      INTEGER,
    table_description VARCHAR,
    table_name        VARCHAR,
    table_index       INTEGER,
    table_title       VARCHAR,
    table_type        VARCHAR,
    table_utype       VARCHAR,
    PRIMARY KEY (ivoid, table_index),
    CONSTRAINT fk_res_table_ivoid
        FOREIGN KEY(ivoid)
            REFERENCES rr.resource(ivoid)
            ON DELETE CASCADE
);

-- Force some fields in lowercase:

CREATE OR REPLACE FUNCTION rr.lower_res_table() RETURNS trigger AS
$BODY$
BEGIN
    NEW.ivoid       := lower(OLD.ivoid);
    NEW.table_name  := lower(OLD.table_name);
    NEW.table_type  := lower(OLD.table_type);
    NEW.table_utype := lower(OLD.table_utype);
END
$BODY$
LANGUAGE plpgsql;

CREATE TRIGGER trg_res_table BEFORE INSERT OR UPDATE ON rr.res_table
    FOR EACH ROW EXECUTE PROCEDURE rr.lower_res_table();

-- Create some indexes:

CREATE INDEX idx_res_table_table_name       ON rr.res_table (table_name);
CREATE INDEX idx_res_table_description_name ON rr.res_table (table_description);

-- Update TAP_SCHEMA:

INSERT INTO "TAP_SCHEMA"."tables" VALUES(6, 'rr', 'rr.res_table', 'table', '(Relational) tables that are part of schemata or resources.', 'xpath:/(tableset/schema/ - )table/');

INSERT INTO "TAP_SCHEMA"."columns" VALUES (1,  'rr.res_table', 'ivoid'            , 'The parent resource.'                                                                                                                                                                                                                                                                                                                       , NULL, NULL, 'xpath:/identifier', 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (2,  'rr.res_table', 'schema_index'     , 'Index of the schema this table belongs to, if it belongs to a schema (otherwise NULL).'                                                                                                                                                                                                                                                     , NULL, NULL, NULL               , 'INTEGER', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (3,  'rr.res_table', 'table_description', 'A free-text description of the table''s contents.'                                                                                                                                                                                                                                                                                          , NULL, NULL, 'xpath:description', 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (4,  'rr.res_table', 'table_name'       , 'The fully qualified name of the table. This name should include all catalog or schema prefixes needed to distinguish it in a query.'                                                                                                                                                                                                        , NULL, NULL, 'xpath:name'       , 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (5,  'rr.res_table', 'table_index'      , 'An arbitrary identifier for the tables belonging to a resource.'                                                                                                                                                                                                                                                                            , NULL, NULL, NULL               , 'INTEGER', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (6,  'rr.res_table', 'table_title'      , 'A descriptive, human-interpretable name for the table.'                                                                                                                                                                                                                                                                                     , NULL, NULL, 'xpath:title'      , 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (7,  'rr.res_table', 'table_type'       , 'A name for the role this table plays. Recognized values include öutput", indicating this table is output from a query; "base_table", indicating a table whose records represent the main subjects of its schema; and "view", indicating that the table represents a useful combination or subset of other tables. Other values are allowed.', NULL, NULL, 'xpath:@type'      , 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (8,  'rr.res_table', 'table_utype'      , 'An identifier for a concept in a data model that the data in this table as a whole represent.'                                                                                                                                                                                                                                              , NULL, NULL, 'xpath:utype'      , 'VARCHAR', -1, 1, 1, 1, NULL);

INSERT INTO "TAP_SCHEMA"."keys" VALUES ('fk_res_table_ivoid', 'rr.res_table', 'rr.resource', NULL, NULL);

INSERT INTO "TAP_SCHEMA"."key_columns" VALUES ('fk_res_table_ivoid', 'ivoid', 'ivoid');

--------------------------------------------------------------------------------

-- Create the table 'table_column':
CREATE TABLE rr.table_column (
    ivoid              VARCHAR,
    table_index        INTEGER,
    name               VARCHAR,
    ucd                VARCHAR,
    unit               VARCHAR,
    utype              VARCHAR,
    std                INTEGER,
    datatype           VARCHAR,
    extended_schema    VARCHAR,
    extended_type      VARCHAR,
    arraysize          VARCHAR,
    delim              VARCHAR,
    type_system        VARCHAR,
    flag               VARCHAR,
    column_description VARCHAR,
    PRIMARY KEY (ivoid, table_index, name),
    CONSTRAINT fk_table_column_ivoid_table_index
        FOREIGN KEY(ivoid, table_index)
            REFERENCES rr.res_table(ivoid, table_index)
            ON DELETE CASCADE
);

-- Force some fields in lowercase:

CREATE OR REPLACE FUNCTION rr.lower_table_column() RETURNS trigger AS
$BODY$
BEGIN
    NEW.ivoid       := lower(OLD.ivoid);
    NEW.name        := lower(OLD.name);
    NEW.ucd         := lower(OLD.ucd);
    NEW.utype       := lower(OLD.table_utype);
    NEW.datatype    := lower(OLD.datatype);
    NEW.type_system := lower(OLD.type_system);
END
$BODY$
LANGUAGE plpgsql;

CREATE TRIGGER trg_table_column BEFORE INSERT OR UPDATE ON rr.table_column
    FOR EACH ROW EXECUTE PROCEDURE rr.lower_table_column();

-- Create some indexes:

CREATE INDEX idx_table_column_description ON rr.table_column (column_description);
CREATE INDEX idx_table_column_name        ON rr.table_column (name);
CREATE INDEX idx_table_column_ucd         ON rr.table_column (ucd);
CREATE INDEX idx_table_column_utype       ON rr.table_column (utype);

-- Update TAP_SCHEMA:

INSERT INTO "TAP_SCHEMA"."tables" VALUES(7, 'rr', 'rr.table_column', 'table', 'Metadata on columns of a resource''s tables.', 'xpath:/(tableset/schema/ - )/table/column/', NULL);

INSERT INTO "TAP_SCHEMA"."columns" VALUES (1,  'rr.table_column', 'ivoid'             , 'The parent resource.'                                                                                                                                                              , NULL, NULL, 'xpath:/identifier'             , 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (2,  'rr.table_column', 'table_index'       , 'Index of the table this column belongs to.'                                                                                                                                        , NULL, NULL, NULL                            , 'INTEGER', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (3,  'rr.table_column', 'name'              , 'The name of the column.'                                                                                                                                                           , NULL, NULL, 'xpath:name'                    , 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (4,  'rr.table_column', 'ucd'               , 'A unified content descriptor that describes the scientific content of the column.'                                                                                                 , NULL, NULL, 'xpath:ucd'                     , 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (5,  'rr.table_column', 'unit'              , 'The unit associated with all values in the column.'                                                                                                                                , NULL, NULL, 'xpath:unit'                    , 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (6,  'rr.table_column', 'utype'             , 'An identifier for a role in a data model that the data in this column represents.'                                                                                                 , NULL, NULL, 'xpath:utype'                   , 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (7,  'rr.table_column', 'std'               , 'If 1, the meaning and use of this column is reserved and defined by a standard model. If 0, it represents a database-specific column that effectively extends beyond the standard.', NULL, NULL, 'xpath:@std'                    , 'INTEGER', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (8,  'rr.table_column', 'datatype'          , 'The type of the data contained in the column.'                                                                                                                                     , NULL, NULL, 'xpath:dataType'                , 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (9,  'rr.table_column', 'extended_schema'   , 'An identifier for the schema that the value given by the extended attribute is drawn from.'                                                                                        , NULL, NULL, 'xpath:dataType/@extendedSchema', 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (10, 'rr.table_column', 'extended_type'     , 'A custom type for the values this column contains.'                                                                                                                                , NULL, NULL, 'xpath:dataType/@extendedType'  , 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (11, 'rr.table_column', 'arraysize'         , 'The shape of the array that constitutes the value, e.g., 4, *, 4*, 5x4, or 5x*, as specified by VOTable.'                                                                          , NULL, NULL, 'xpath:arraysize'               , 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (12, 'rr.table_column', 'delim'             , 'The string that is used to delimit elements of an array value when arraysize is not ''1''.'                                                                                        , NULL, NULL, 'xpath:dataType/@delim'         , 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (13, 'rr.table_column', 'type_system'       , 'The type system used, as a QName with a canonical prefix; this will ususally be one of vs:simpledatatype, vs:votabletype, and vs:taptype.'                                         , NULL, NULL, 'xpath:dataType/@xsi:type'      , 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (14, 'rr.table_column', 'flag'              , 'Hash-separated keywords representing traits of the column. Recognized values include ïndexed", "primary", and "nullable".'                                                         , NULL, NULL, 'xpath:flag'                    , 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (15, 'rr.table_column', 'column_description', 'A free-text description of the column''s contents.'                                                                                                                                , NULL, NULL, 'xpath:description'             , 'VARCHAR', -1, 1, 1, 1, NULL);

INSERT INTO "TAP_SCHEMA"."keys" VALUES ('fk_table_column_ivoid_table_index', 'rr.table_column', 'rr.res_table', NULL, NULL);

INSERT INTO "TAP_SCHEMA"."key_columns" VALUES ('fk_table_column_ivoid_table_index', 'ivoid'      , 'ivoid');
INSERT INTO "TAP_SCHEMA"."key_columns" VALUES ('fk_table_column_ivoid_table_index', 'table_index', 'table_index');

--------------------------------------------------------------------------------

-- Create the table 'interface':
CREATE TABLE rr.interface (
    ivoid              VARCHAR,
    cap_index          INTEGER,
    intf_index         INTEGER,
    intf_type          VARCHAR,
    intf_role          VARCHAR,
    std_version        VARCHAR,
    query_type         VARCHAR,
    result_type        VARCHAR,
    wsdl_url           VARCHAR,
    url_use            VARCHAR,
    access_url         VARCHAR,
    mirror_url         VARCHAR,
    authenticated_only INTEGER,
    PRIMARY KEY (ivoid, intf_index),
    CONSTRAINT fk_interface_ivoid_cap_index
        FOREIGN KEY(ivoid, cap_index)
            REFERENCES rr.capability(ivoid, cap_index)
            ON DELETE CASCADE
);

-- Force some fields in lowercase:

CREATE OR REPLACE FUNCTION rr.lower_interface() RETURNS trigger AS
$BODY$
BEGIN
    NEW.ivoid       := lower(OLD.ivoid);
    NEW.intf_type   := lower(OLD.intf_type);
    NEW.intf_role   := lower(OLD.intf_role);
    NEW.std_version := lower(OLD.std_version);
    NEW.query_type  := lower(OLD.query_type);
    NEW.result_type := lower(OLD.result_type);
    NEW.url_use     := lower(OLD.url_use);
END
$BODY$
LANGUAGE plpgsql;

CREATE TRIGGER trg_interface BEFORE INSERT OR UPDATE ON rr.interface
    FOR EACH ROW EXECUTE PROCEDURE rr.lower_interface();

-- Create some indexes:

CREATE INDEX idx_interface_intf_type ON rr.interface (intf_type);

-- Update TAP_SCHEMA:

INSERT INTO "TAP_SCHEMA"."tables" VALUES(8, 'rr', 'rr.interface', 'table', 'Information on access modes of a capability.', 'xpath:/capability/interface/', NULL);

INSERT INTO "TAP_SCHEMA"."columns" VALUES (1,  'rr.interface', 'ivoid'             , 'The parent resource.'                                                                                                                                                                                                                                             , NULL, NULL, 'xpath:/identifier'   , 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (2,  'rr.interface', 'cap_index'         , 'The index of the parent capability.'                                                                                                                                                                                                                              , NULL, NULL, NULL                  , 'INTEGER', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (3,  'rr.interface', 'intf_index'        , 'An arbitrary identifier for the interfaces of a resource.'                                                                                                                                                                                                        , NULL, NULL, NULL                  , 'INTEGER', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (4,  'rr.interface', 'intf_type'         , 'The type of the interface (vr:webbrowser, vs:paramhttp, etc).'                                                                                                                                                                                                    , NULL, NULL, 'xpath:@xsi:type'     , 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (5,  'rr.interface', 'intf_role'         , 'An identifier for the role the interface plays in the particular capability. If the value is equal to ßtd" or begins with ßtd:", then the interface refers to a standard interface defined by the standard referred to by the capability''s standardID attribute.', NULL, NULL, 'xpath:@role'         , 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (6,  'rr.interface', 'std_version'       , 'The version of a standard interface specification that this interface complies with. When the interface is provided in the context of a Capability element, then the standard being refered to is the one identified by the Capability''s standardID element.'    , NULL, NULL, 'xpath:@version'      , 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (7,  'rr.interface', 'query_type'        , 'Hash-joined list of expected HTTP method (get or post) supported by the service.'                                                                                                                                                                                 , NULL, NULL, 'xpath:queryType'     , 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (8,  'rr.interface', 'result_type'       , 'The MIME type of a document returned in the HTTP response.'                                                                                                                                                                                                       , NULL, NULL, 'xpath:resultType'    , 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (9,  'rr.interface', 'wsdl_url'          , 'The location of the WSDL that describes this Web Service. If NULL, the location can be assumed to be the accessURL with ''?wsdl'' appended.'                                                                                                                      , NULL, NULL, 'xpath:wsdlURL'       , 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (10, 'rr.interface', 'url_use'           , 'A flag indicating whether this should be interpreted as a base URL (''base''), a full URL (''full''), or a URL to a directory that will produce a listing of files (''dir'').'                                                                                    , NULL, NULL, 'xpath:accessURL/@use', 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (11, 'rr.interface', 'access_url'        , 'The URL at which the interface is found.'                                                                                                                                                                                                                         , NULL, NULL, 'xpath:accessURL'     , 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (12, 'rr.interface', 'mirror_url'        , 'Secondary access URLs of this interface, separated by hash characters.'                                                                                                                                                                                           , NULL, NULL, 'xpath:mirrorURL'     , 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (13, 'rr.interface', 'authenticated_only', 'A flag for whether an interface is available for anonymous use (=0) or only authenticated clients are served (=1).'                                                                                                                                               , NULL, NULL, NULL                  , 'INTEGER', -1, 1, 1, 1, NULL);

INSERT INTO "TAP_SCHEMA"."keys" VALUES ('fk_interface_ivoid_cap_index', 'rr.interface', 'rr.capability', NULL, NULL);

INSERT INTO "TAP_SCHEMA"."key_columns" VALUES ('fk_interface_ivoid_cap_index', 'ivoid'    , 'ivoid');
INSERT INTO "TAP_SCHEMA"."key_columns" VALUES ('fk_interface_ivoid_cap_index', 'cap_index', 'cap_index');

--------------------------------------------------------------------------------

-- Create the table 'intf_param':
CREATE TABLE rr.intf_param (
    ivoid             VARCHAR,
    intf_index        INTEGER,
    name              VARCHAR,
    ucd               VARCHAR,
    unit              VARCHAR,
    utype             VARCHAR,
    std               INTEGER,
    datatype          VARCHAR,
    extended_schema   VARCHAR,
    extended_type     VARCHAR,
    arraysize         VARCHAR,
    delim             VARCHAR,
    param_use         VARCHAR,
    param_description VARCHAR,
    PRIMARY KEY (ivoid, intf_index, name),
    CONSTRAINT fk_intf_param_ivoid_intf_index
        FOREIGN KEY(ivoid, intf_index)
            REFERENCES rr.interface(ivoid, intf_index)
            ON DELETE CASCADE
);

-- Update TAP_SCHEMA:

INSERT INTO "TAP_SCHEMA"."tables" VALUES(9, 'rr', 'rr.intf_param', 'table', 'Input parameters for services.', 'xpath:/capability/interface/param/', NULL);

INSERT INTO "TAP_SCHEMA"."columns" VALUES (1,  'rr.intf_param', 'ivoid'            , 'The parent resource.'                                                                                                                                                                    , NULL, NULL, 'xpath:/identifier'             , 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (2,  'rr.intf_param', 'intf_index'       , 'The index of the interface this parameter belongs to.'                                                                                                                                   , NULL, NULL, NULL                            , 'INTEGER', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (3,  'rr.intf_param', 'name'             , 'The name of the parameter.'                                                                                                                                                              , NULL, NULL, 'xpath:name'                    , 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (4,  'rr.intf_param', 'ucd'              , 'A unified content descriptor that describes the scientific content of the parameter.'                                                                                                    , NULL, NULL, 'xpath:ucd'                     , 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (5,  'rr.intf_param', 'unit'             , 'The unit associated with all values in the parameter.'                                                                                                                                   , NULL, NULL, 'xpath:unit'                    , 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (6,  'rr.intf_param', 'utype'            , 'An identifier for a role in a data model that the data in this parameter represents.'                                                                                                    , NULL, NULL, 'xpath:utype'                   , 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (7,  'rr.intf_param', 'std'              , 'If 1, the meaning and use of this parameter is reserved and defined by a standard model. If 0, it represents a database-specific parameter that effectively extends beyond the standard.', NULL, NULL, 'xpath:std'                     , 'INTEGER', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (8,  'rr.intf_param', 'datatype'         , 'The type of the data contained in the parameter.'                                                                                                                                        , NULL, NULL, 'xpath:dataType'                , 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (9,  'rr.intf_param', 'extended_schema'  , 'An identifier for the schema that the value given by the extended attribute is drawn from.'                                                                                              , NULL, NULL, 'xpath:dataType/@extendedSchema', 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (10, 'rr.intf_param', 'extended_type'    , 'A custom type for the values this parameter contains.'                                                                                                                                   , NULL, NULL, 'xpath:dataType/@extendedType'  , 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (11, 'rr.intf_param', 'arraysize'        , 'The shape of the array that constitutes the value, e.g., 4, *, 4*, 5x4, or 5x*, as specified by VOTable.'                                                                                , NULL, NULL, 'xpath:dataType/@arraysize'     , 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (12, 'rr.intf_param', 'delim'            , 'The string that is used to delimit elements of an array value when arraysize is not ''1''.'                                                                                              , NULL, NULL, 'xpath:dataType/@delim'         , 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (13, 'rr.intf_param', 'param_use'        , 'An indication of whether this parameter is required to be provided for the application or service to work properly (one of required, optional, ignored, or NULL).'                       , NULL, NULL, 'xpath:@use'                    , 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (14, 'rr.intf_param', 'param_description', 'A free-text description of the parameter''s contents.'                                                                                                                                   , NULL, NULL, 'xpath:description'             , 'VARCHAR', -1, 1, 1, 1, NULL);

INSERT INTO "TAP_SCHEMA"."keys" VALUES ('fk_intf_param_ivoid_intf_index', 'rr.intf_param', 'rr.interface', NULL, NULL);

INSERT INTO "TAP_SCHEMA"."key_columns" VALUES ('fk_intf_param_ivoid_intf_index', 'ivoid'     , 'ivoid');
INSERT INTO "TAP_SCHEMA"."key_columns" VALUES ('fk_intf_param_ivoid_intf_index', 'intf_index', 'intf_index');

--------------------------------------------------------------------------------

-- Create the table 'relationship':
CREATE TABLE rr.relationship (
    ivoid             VARCHAR,
    relationship_type VARCHAR,
    related_id        VARCHAR,
    related_name      VARCHAR,
    PRIMARY KEY (ivoid, related_id),
    CONSTRAINT fk_relationship_ivoid
        FOREIGN KEY(ivoid)
            REFERENCES rr.resource(ivoid)
            ON DELETE CASCADE
);

-- Force some fields in lowercase:

CREATE OR REPLACE FUNCTION rr.lower_relationship() RETURNS trigger AS
$BODY$
BEGIN
    NEW.ivoid             := lower(OLD.ivoid);
    NEW.relationship_type := lower(OLD.relationship_type);
    NEW.related_id        := lower(OLD.related_id);
END
$BODY$
LANGUAGE plpgsql;

CREATE TRIGGER trg_relationship BEFORE INSERT OR UPDATE ON rr.relationship
    FOR EACH ROW EXECUTE PROCEDURE rr.lower_relationship();

-- Update TAP_SCHEMA:

INSERT INTO "TAP_SCHEMA"."tables" VALUES(10, 'rr', 'rr.relationship', 'table', 'Relationships between resources (like mirroring, derivation, serving a data collection).', 'xpath:/content/relationship/', NULL);

INSERT INTO "TAP_SCHEMA"."columns" VALUES (1,  'rr.relationship', 'ivoid'            , 'The parent resource.'                                                                                         , NULL, NULL, 'xpath:/identifier'            , 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (2,  'rr.relationship', 'relationship_type', 'The type of the relationship; these terms are drawn from a controlled vocabulary and are DataCite-compatible.', NULL, NULL, 'xpath:relationshipType'       , 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (3,  'rr.relationship', 'related_id'       , 'The IVOA identifier for the resource referred to.'                                                            , NULL, NULL, 'xpath:relatedResource/@ivo-id', 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (4,  'rr.relationship', 'related_name'     , 'The name of resource that this resource is related to.'                                                       , NULL, NULL, 'xpath:relatedResource'        , 'VARCHAR', -1, 1, 1, 1, NULL);

INSERT INTO "TAP_SCHEMA"."keys" VALUES ('fk_relationship_ivoid', 'rr.relationship', 'rr.resource', NULL, NULL);

INSERT INTO "TAP_SCHEMA"."key_columns" VALUES ('fk_relationship_ivoid', 'ivoid', 'ivoid');

--------------------------------------------------------------------------------

-- Create the table 'validation':
CREATE TABLE rr.validation (
    ivoid        VARCHAR,
    validated_by VARCHAR,
    val_level    INTEGER,
    cap_index    INTEGER,
    CONSTRAINT fk_validation_ivoid
        FOREIGN KEY(ivoid)
            REFERENCES rr.resource(ivoid)
            ON DELETE CASCADE
);

-- Force some fields in lowercase:

CREATE OR REPLACE FUNCTION rr.lower_validation() RETURNS trigger AS
$BODY$
BEGIN
    NEW.ivoid        := lower(OLD.ivoid);
    NEW.validated_by := lower(OLD.validated_by);
END
$BODY$
LANGUAGE plpgsql;

CREATE TRIGGER trg_validation BEFORE INSERT OR UPDATE ON rr.validation
    FOR EACH ROW EXECUTE PROCEDURE rr.lower_interface();

-- Update TAP_SCHEMA:

INSERT INTO "TAP_SCHEMA"."tables" VALUES(11, 'rr', 'rr.validation', 'table', 'Validation levels for resources and capabilities.', 'xpath:/(capability/ - )validationLevel', NULL);

INSERT INTO "TAP_SCHEMA"."columns" VALUES (1,  'rr.validation', 'ivoid'       , 'The parent resource.'                                                                                                                                                                                         , NULL, NULL, 'xpath:/identifier'                 , 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (2,  'rr.validation', 'validated_by', 'The IVOA ID of the registry or organisation that assigned the validation level.'                                                                                                                              , NULL, NULL, 'xpath:validationLevel/@validatedBy', 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (3,  'rr.validation', 'val_level'   , 'A numeric grade describing the quality of the resource description, when applicable, to be used to indicate the confidence an end-user can put in the resource as part of a VO application or research study.', NULL, NULL, 'xpath:validationLevel'             , 'INTEGER', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (4,  'rr.validation', 'cap_index'   , 'If non-NULL, the validation only refers to the capability referenced here.'                                                                                                                                   , NULL, NULL, NULL                                , 'INTEGER', -1, 1, 1, 1, NULL);

INSERT INTO "TAP_SCHEMA"."keys" VALUES ('fk_validation_ivoid', 'rr.validation', 'rr.resource', NULL, NULL);

INSERT INTO "TAP_SCHEMA"."key_columns" VALUES ('fk_validation_ivoid', 'ivoid', 'ivoid');

--------------------------------------------------------------------------------

-- Create the table 'res_date':
CREATE TABLE rr.res_date (
    ivoid      VARCHAR,
    date_value TIMESTAMP,
    value_role VARCHAR,
    CONSTRAINT fk_res_date_ivoid
        FOREIGN KEY(ivoid)
            REFERENCES rr.resource(ivoid)
            ON DELETE CASCADE
);

-- Force some fields in lowercase:

CREATE OR REPLACE FUNCTION rr.lower_res_date() RETURNS trigger AS
$BODY$
BEGIN
    NEW.ivoid      := lower(OLD.ivoid);
    NEW.value_role := lower(OLD.value_role);
END
$BODY$
LANGUAGE plpgsql;

CREATE TRIGGER trg_res_date BEFORE INSERT OR UPDATE ON rr.res_date
    FOR EACH ROW EXECUTE PROCEDURE rr.lower_res_date();

-- Update TAP_SCHEMA:

INSERT INTO "TAP_SCHEMA"."tables" VALUES(12, 'rr', 'rr.res_date', 'table', 'A date associated with an event in the life cycle of the resource.', 'xpath:/curation/', NULL);

INSERT INTO "TAP_SCHEMA"."columns" VALUES (1,  'rr.res_date', 'ivoid'     , 'The parent resource.'                                                                                                                          , NULL, NULL, 'xpath:/identifier', 'VARCHAR'  , -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (2,  'rr.res_date', 'date_value', 'A date associated with an event in the life cycle of the resource.'                                                                            , NULL, NULL, 'xpath:date'       , 'TIMESTAMP', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (3,  'rr.res_date', 'value_role', 'A string indicating what the date refers to, e.g., created, availability, updated. This value is generally drawn from a controlled vocabulary.', NULL, NULL, 'xpath:date/@role' , 'VARCHAR'  , -1, 1, 1, 1, NULL);

INSERT INTO "TAP_SCHEMA"."keys" VALUES ('fk_res_date_ivoid', 'rr.res_date', 'rr.resource', NULL, NULL);

INSERT INTO "TAP_SCHEMA"."key_columns" VALUES ('fk_res_date_ivoid', 'ivoid', 'ivoid');

--------------------------------------------------------------------------------

-- Create the table 'res_detail':
CREATE TABLE rr.res_detail (
    ivoid        VARCHAR,
    cap_index    INTEGER,
    detail_xpath VARCHAR,
    detail_value VARCHAR,
    CONSTRAINT fk_res_detail_ivoid
        FOREIGN KEY(ivoid)
            REFERENCES rr.resource(ivoid)
            ON DELETE CASCADE
);

-- Force some fields in lowercase:

CREATE OR REPLACE FUNCTION rr.lower_res_detail() RETURNS trigger AS
$BODY$
BEGIN
    NEW.ivoid := lower(OLD.ivoid);
END
$BODY$
LANGUAGE plpgsql;

CREATE TRIGGER trg_res_detail BEFORE INSERT OR UPDATE ON rr.res_detail
    FOR EACH ROW EXECUTE PROCEDURE rr.lower_res_detail();

-- Create some indexes:

CREATE INDEX idx_res_detail_xpath ON rr.res_detail (detail_xpath);
CREATE INDEX idx_res_detail_value ON rr.res_detail (detail_value);

-- Update TAP_SCHEMA:

INSERT INTO "TAP_SCHEMA"."tables" VALUES(13, 'rr', 'rr.res_detail', 'table', 'XPath-value pairs for members of resource or capability and their derivations that are less used and/or from VOResource extensions.', NULL, NULL);

INSERT INTO "TAP_SCHEMA"."columns" VALUES (1,  'rr.res_detail', 'ivoid'       , 'The parent resource.'                                                                                       , NULL, NULL, 'xpath:/identifier', 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (2,  'rr.res_detail', 'cap_index'   , 'The index of the parent capability; if NULL the xpath-value pair describes a member of the entire resource.', NULL, NULL, NULL               , 'INTEGER', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (3,  'rr.res_detail', 'detail_xpath', 'The xpath of the data item.'                                                                                , NULL, NULL, NULL               , 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (4,  'rr.res_detail', 'detail_value', '(Atomic) value of the member.'                                                                              , NULL, NULL, NULL               , 'VARCHAR', -1, 1, 1, 1, NULL);

INSERT INTO "TAP_SCHEMA"."keys" VALUES ('fk_res_detail_ivoid', 'rr.res_detail', 'rr.resource', NULL, NULL);

INSERT INTO "TAP_SCHEMA"."key_columns" VALUES ('fk_res_detail_ivoid', 'ivoid', 'ivoid');

--------------------------------------------------------------------------------

-- Create the table 'alt_identifier':
CREATE TABLE rr.alt_identifier (
    ivoid          VARCHAR,
    alt_identifier VARCHAR,
    CONSTRAINT fk_alt_identifier_ivoid
        FOREIGN KEY(ivoid)
            REFERENCES rr.resource(ivoid)
            ON DELETE CASCADE
);

-- Create some indexes:

CREATE INDEX idx_alt_identifier ON rr.alt_identifier (alt_identifier);

-- Update TAP_SCHEMA:

INSERT INTO "TAP_SCHEMA"."tables" VALUES(14, 'rr', 'rr.alt_identifier', 'table', 'An alternate identifier associated with this record.', 'xpath:/(curation/creator/ - )altIdentifier', NULL);

INSERT INTO "TAP_SCHEMA"."columns" VALUES (1,  'rr.alt_identifier', 'ivoid'         , 'The parent resource.'                                                            , NULL, NULL, 'xpath:/identifier', 'VARCHAR', -1, 1, 1, 1, NULL);
INSERT INTO "TAP_SCHEMA"."columns" VALUES (2,  'rr.alt_identifier', 'alt_identifier', 'An identifier for the resource or an entity related to the resource in URI form.', NULL, NULL, NULL               , 'VARCHAR', -1, 1, 1, 1, NULL);

INSERT INTO "TAP_SCHEMA"."keys" VALUES ('fk_alt_identifier_ivoid', 'rr.alt_identifier', 'rr.resource', NULL, NULL);

INSERT INTO "TAP_SCHEMA"."key_columns" VALUES ('fk_alt_identifier_ivoid', 'ivoid', 'ivoid');

\echo '[INFO: rr tables created]'
