#!/bin/bash
set -e

# Add the TAP user to the database:
psql -v ON_ERROR_STOP=1 --username "$POSTGRES_USER" --dbname "$POSTGRES_DB" <<-EOSQL
    -- Create TAP user:
    CREATE USER $TAPUSER_NAME;
    ALTER USER $TAPUSER_NAME WITH ENCRYPTED PASSWORD '$TAPUSER_PASSWORD';
    ALTER USER $TAPUSER_NAME WITH LOGIN;

    -- Give the appropriate permissions for the user '$TAPUSER_NAME':
    GRANT CONNECT ON DATABASE $DB_NAME TO $TAPUSER_NAME;
    REVOKE ALL ON SCHEMA public, rr, "TAP_SCHEMA", "TAP_UPLOAD" FROM $TAPUSER_NAME;
    GRANT USAGE ON SCHEMA public, rr, "TAP_SCHEMA" TO $TAPUSER_NAME;
    GRANT SELECT ON ALL TABLES IN SCHEMA public, rr, "TAP_SCHEMA" TO $TAPUSER_NAME;
    GRANT ALL ON SCHEMA "TAP_UPLOAD" TO $TAPUSER_NAME;
    GRANT ALL ON ALL TABLES IN SCHEMA "TAP_UPLOAD" TO $TAPUSER_NAME;
EOSQL

# Allow only a local access (so, from inside the container),
# execpt for the user 'tapuser':
cat >$PGDATA/pg_hba.conf << EOF
# TYPE  DATABASE        USER            ADDRESS                 METHOD

# "local" is for Unix domain socket connections only
local   all             all                                     trust

# IPv4 local connections:
host    all             all             127.0.0.1/32            trust

# IPv6 local connections:
host    all             all             ::1/128                 trust

# Allow the user 'tapuser' to access the database:
host    $DB_NAME    $TAPUSER_NAME    all    md5
EOF
